import { ArrayRga } from '../../types/rga-array/ArrayRga';
import { BinaryRga } from '../../types/rga-binary/BinaryRga';
import { Const } from '../../types/const/Const';
import { ObjectLww } from '../../types/lww-object/ObjectLww';
import { Path } from '../../../json-pointer';
import { StringRga } from '../../types/rga-string/StringRga';
import { ValueLww } from '../../types/lww-value/ValueLww';
import { ArrayLww } from '../../types/lww-array/ArrayLww';
import { ExtensionApi, ExtensionDefinition, ExtensionJsonNode } from '../../extensions/types';
import { NodeEvents } from './events/NodeEvents';
import type * as types from '../proxy/types';
import type { JsonNode, JsonNodeView } from '../../types';
import type { ModelApi } from './ModelApi';
import type { Printable } from '../../../util/print/types';
export type ApiPath = string | number | Path | void;
export declare class NodeApi<N extends JsonNode = JsonNode> implements Printable {
    readonly node: N;
    readonly api: ModelApi<any>;
    constructor(node: N, api: ModelApi<any>);
    private ev;
    get events(): NodeEvents;
    find(path?: ApiPath): JsonNode;
    in(path?: ApiPath): ValueApi<ValueLww<any>>;
    asVal(): ValueApi;
    asStr(): StringApi;
    asBin(): BinaryApi;
    asArr(): ArrayApi;
    asTup(): TupleApi;
    asObj(): ObjectApi;
    asConst(): ConstApi;
    asExt<EN extends ExtensionJsonNode, V, EApi extends ExtensionApi<EN>>(ext: ExtensionDefinition<any, EN, EApi>): EApi;
    val(path?: ApiPath): ValueApi;
    str(path?: ApiPath): StringApi;
    bin(path?: ApiPath): BinaryApi;
    arr(path?: ApiPath): ArrayApi;
    tup(path?: ApiPath): TupleApi;
    obj(path?: ApiPath): ObjectApi;
    const(path?: ApiPath): ConstApi;
    view(): JsonNodeView<N>;
    toString(tab?: string): string;
}
export declare class ConstApi<N extends Const<any> = Const<any>> extends NodeApi<N> {
    proxy(): types.ProxyNodeConst<N>;
}
export declare class ValueApi<N extends ValueLww<any> = ValueLww<any>> extends NodeApi<N> {
    set(json: JsonNodeView<N>): this;
    proxy(): types.ProxyNodeVal<N>;
}
export declare class TupleApi<N extends ArrayLww<any> = ArrayLww<any>> extends NodeApi<N> {
    set(entries: [index: number, value: unknown][]): this;
    proxy(): types.ProxyNodeVec<N>;
}
export declare class ObjectApi<N extends ObjectLww<any> = ObjectLww<any>> extends NodeApi<N> {
    set(entries: Partial<JsonNodeView<N>>): this;
    del(keys: string[]): this;
    proxy(): types.ProxyNodeObj<N>;
}
export declare class StringApi extends NodeApi<StringRga> {
    ins(index: number, text: string): this;
    del(index: number, length: number): this;
    proxy(): types.ProxyNodeStr;
}
export declare class BinaryApi extends NodeApi<BinaryRga> {
    ins(index: number, data: Uint8Array): this;
    del(index: number, length: number): this;
    proxy(): types.ProxyNodeBin;
}
export declare class ArrayApi<N extends ArrayRga<any> = ArrayRga<any>> extends NodeApi<N> {
    ins(index: number, values: Array<JsonNodeView<N>[number]>): this;
    del(index: number, length: number): this;
    length(): number;
    proxy(): types.ProxyNodeArr<N>;
}
